#!/bin/bash
#
###############################################################
#
# LiveCD customization script
# 
# Run this script on a SL4 Installation
# to prepare the system for a LiveCD 
#
# Urs Beyerle, PSI
#
###############################################################
#
###############################################################

function usage() {

   ## Usage
   # ----------------------------------------------------------

   cat <<EOF

   Options:

    -h:       print this screen
    -psi:     customize for PSI Live CD

EOF

}

###############################################################

###############################################################
# Definitions
###############################################################

### read options from command-line
PSI=""
while [ $# -gt 0 ]; do

    case "$1" in
       -h)
            usage; exit;;
       -psi)
            PSI=true; shift; continue;;
       *)
            usage; exit;;
    esac

done


### arch x86_64 or i686 ?

ARCH=$( /bin/arch )
[ "$ARCH" != "x86_64" ] && ARCH=i686



###############################################################
# Configure system
###############################################################

# delete users (just to be sure)
userdel -r psi 2>/dev/null
userdel -r sluser 2>/dev/null
userdel -r user 2>/dev/null

# move /opt to /usr/opt and link /opt -> /usr/opt
# (if not already yet done before)
if [ ! -L /opt ]; then
    echo "Move /opt to /usr/opt"
    mv /opt /usr/
    ln -s /usr/opt /opt
fi
    
# make sure we have a original copy of /etc/rc.d/rc.sysinit
if [ ! -e /etc/rc.d/rc.sysinit.ori ]; then
    echo "Save /etc/rc.d/rc.sysinit as /etc/rc.d/rc.sysinit.ori"
    cp -a /etc/rc.d/rc.sysinit /etc/rc.d/rc.sysinit.ori
fi

# copy back /etc/yum.repos.d
if [ ! $PSI ]; then
    cp -a /etc/yum.repos.d.ori /etc/yum.repos.d
fi

# run cfagent
if [ $PSI ]; then
    # cfagent needs running afs
    /etc/init.d/afs start
    echo "Run cfagent ..."
    cfagent
    echo "done."
fi

# run slocate
echo "Run updatedb..."
. /etc/updatedb.conf
/usr/bin/updatedb -e /tmp,/boot,/livecd,/home
echo "done."

# run prelink
echo "Run prelink..."
if [ -x /etc/cron.daily/prelink ]; then
    echo "/etc/cron.daily/prelink"
    /etc/cron.daily/prelink
fi
if [ -x /etc/cron_backup/prelink ]; then
    echo "/etc/cron_backup/prelink"
    /etc/cron_backup/prelink
fi
echo "done."

# run makewhatis
echo "Run makewhatis..."
makewhatis -u -w
echo "done."

# run prelink


###############################################################
# Clean up and delete files
###############################################################

echo "Cleaning up ..."

### set LANG
export LANG=C

### stop AFS
/etc/init.d/afs stop

### clean up yum
# if ! mount | grep -q /mnt/master; then mount /mnt/master; fi
yum clean all
# umount /mnt/master

### clean log and rpm database
rm -rf /var/log/*
rm -rf /var/lib/rpm/__db.*

### clean /var/spool/mail
rm -rf /var/spool/mail/*
touch /var/spool/mail/root
chmod 600 /var/spool/mail/root

### stop sysstat and create empty log
rpm -q sysstat > /dev/null
if [ "$?" = "0" ]; then
    /etc/init.d/sysstat stop
    mkdir -p /var/log/sa
fi

### create some empty log dirs
rpm -q gdm > /dev/null
[ "$?" = "0" ] && mkdir -p /var/log/gdm
rpm -q iptraf > /dev/null
[ "$?" = "0" ] && mkdir -p /var/log/iptraf
rpm -q samba-common > /dev/null
[ "$?" = "0" ] && mkdir -p /var/log/samba
rpm -q ppp > /dev/null
[ "$?" = "0" ] && mkdir -p /var/log/ppp
rpm -q isdn4k-utils > /dev/null
[ "$?" = "0" ] && mkdir -p /var/log/vbox
rpm -q cups > /dev/null
[ "$?" = "0" ] && mkdir -p /var/log/cups

### delete .cfsaved Files
if [ $PSI ]; then
    echo -n "Delete .cfsaved Files, please wait: "
    find / -name "*\.cfsaved" -exec rm -f {} \;
    echo "ok."
fi

### delete .rpmori .rpm Files in /etc
find /etc -name "*\.rpmorig" -exec rm -f {} \;

### clean up /root
rm -f /root/.bash_history

### clean AFS cache
if [ -d /usr/vice/cache ]; then
    rm -rf /usr/vice/cache
    mkdir /usr/vice/cache
fi
if [ -d /var/cache/openafs ]; then
    rm -rf /var/cache/openafs
    mkdir /var/cache/openafs
fi

### clean up /var/tmp
rm -rf /var/tmp/*

### remove root passwd
sed -i "s|^root:.*|root:\*:12943:0:99999:7:::|" /etc/shadow

### Remove /.autofsck
rm -rf /.autofsck

### disable cfagent
rm -f /etc/cron.d/cfengine

### disable check_update in rc.local.psi
if [ $PSI ]; then
    sed -i "s|/etc/init.d/check_update|# /etc/init.d/check_update|" /etc/rc.d/rc.local.psi
fi

### create link to running kernel
ln -fs /boot/vmlinuz-$( uname -r ) /boot/vmlinuz

### Files to remove
FILES_REMOVE="/etc/ssh/ssh_host_key \
              /etc/ssh/ssh_host_key.pub \
              /etc/ssh/ssh_host_rsa_key.pub \
              /etc/ssh/ssh_host_dsa_key \
              /etc/ssh/ssh_host_dsa_key.pub"
for file in $FILES_REMOVE; do
    rm -rf $file
done

### remove useless cronjobs
mkdir -p /etc/cron_backup
mv /etc/cron.daily/00-makewhatis.cron /etc/cron_backup/ 2>/dev/null
mv /etc/cron.daily/rpm /etc/cron_backup/ 2>/dev/null
mv /etc/cron.daily/slocate.cron /etc/cron_backup/ 2>/dev/null
mv /etc/cron.daily/tetex.cron /etc/cron_backup/ 2>/dev/null
mv /etc/cron.daily/yum.cron /etc/cron_backup/ 2>/dev/null
mv /etc/cron.daily/prelink /etc/cron_backup/ 2>/dev/null

echo "done."

###############################################################
# Modify files
###############################################################

echo "Modify files ..."

### disable umount of loop device during shutdown
#   this is done really dirty at the moment:
#    -> just replace "loop" with non existing "lo_fake"
sed -i "s|/loop/|/lo_fake/|g" /etc/init.d/netfs
sed -i "s|/loop/|/lo_fake/|g" /etc/init.d/autofs
sed -i "s|/loop/|/lo_fake/|g" /etc/init.d/halt
#    -> instead of bring down lo, bring down eth0 once again.
sed -i "s|ifdown ifcfg-lo|ifdown ifcfg-eth0|g" /etc/init.d/network

### copy new /etc/init.d/halt 
cp -a customize/halt /etc/init.d/halt

### in /etc/rc.d/rc.sysinit
#   comment out 'initlog -c "fsck -T -a $rootdev $fsckoptions"' 
#   to disable fsck of root filesystem
sed -i "s|\tfsck -T -a \$rootdev|\tsleep 0; #fsck -T -a \$rootdev|" /etc/rc.d/rc.sysinit
sed -i "s|\tinitlog -c \"fsck -T -a \$rootdev|\tsleep 0; #initlog -c \"fsck -T -a \$rootdev|" /etc/rc.d/rc.sysinit
#   disable "Remounting root filesystem in read-write mode"
sed -i "s| action \$\"Remounting root filesystem|#action \$\"Remounting root filesystem|" /etc/rc.d/rc.sysinit

### start afs with option -memcache !
if [ $PSI ]; then
    sed -i "s|^EXTRA_OPTIONS=.*|EXTRA_OPTIONS='-fakestat -memcache'|" /etc/sysconfig/afs
else
    grep -q "\-fakestat \-memcache" /etc/sysconfig/afs
    if [ "$?" != "0" ]; then
	sed -i "s|-fakestat|-fakestat -memcache|" /etc/sysconfig/afs
    fi
fi

### source /etc/sysconfig/cfengine
[ $PSI ] && . /etc/sysconfig/cfengine

### Set /etc/motd
if [ "$ARCH" = "x86_64" ]; then
    LiveCD="LiveCD 64bit"
else
    LiveCD="LiveCD"
fi

if [ $PSI ]; then
    echo "Welcome to PSI ${LiveCD} (${CLASS} ${SUBCLASS} SL${RELEASE})" > /etc/motd
else
    echo "Welcome to SL4 ${LiveCD}" > /etc/motd
fi

### Set hostname to psi or slinux (not really necessary)
if [ $PSI ]; then
    HOSTNAME="psi"
    sed -i "s/hostname=.*/hostname=${HOSTNAME}.psi.ch/" /etc/ssmtp/ssmtp.conf
    sed -i "s/HOSTNAME=.*/HOSTNAME=${HOSTNAME}/" /etc/sysconfig/cfengine
else
    HOSTNAME="slinux"
fi  

# change hostname
sed -i "s/HOSTNAME=.*/HOSTNAME=${HOSTNAME}/" /etc/sysconfig/network
sed -i "s/DHCP_HOSTNAME=.*/DHCP_HOSTNAME==${HOSTNAME}/" /etc/sysconfig/networking/devices/ifcfg-eth0

### Modify /etc/redhat-release
ADD=" - LiveCD"
grep -q "$ADD" /etc/redhat-release
if [ "$?" != "0" ]; then
    echo "$( cat /etc/redhat-release )${ADD}" > /etc/redhat-release
fi

### Make scratch
rm -rf /home/scratch
mkdir -p /home/scratch
chmod 1777 /home/scratch
rm -rf /scratch
ln -s /home/scratch /scratch

### set default runlevel to 5
# NOTE: "kudzu-auto" and "cfengine-init" are in runlevel 5
sed -i 's/id:3:initdefault:/id:5:initdefault:/' /etc/inittab

### default desktop
echo 'DESKTOP="KDE"' > /etc/sysconfig/desktop

echo "done."

###############################################################
# Add-ons
###############################################################

echo "Add-ons ..."

if [ ! -e /usr/share/apps/kdesktop/DesktopLinks/System.desktop ]; then
    cp -a customize/System.desktop /usr/share/apps/kdesktop/DesktopLinks/
fi

echo "done."

###############################################################
# Create special files
###############################################################

echo "Create special files ..."

###############################################################
# /etc/rc.d/init.d/kudzu-auto
### Noninteractive HW detection and configuration
cat > /etc/rc.d/init.d/kudzu-auto <<EOF
#!/bin/bash
# Noninteractive HW detection and configuration
. /etc/init.d/functions
action $"Hardware auto-detection, please wait... " /usr/sbin/kudzu -q 
EOF
chmod +x /etc/rc.d/init.d/kudzu-auto
ln -sf /etc/rc.d/init.d/kudzu-auto /etc/rc.d/rc5.d/S04kudzu-auto


###############################################################

### copy own scripts
scripts="mk_xconfig XConfig_Device.sh XConfig_Monitor.sh XConfig_Screen.sh"
for script in $scripts; do
    cp -a customize/${script} /usr/sbin/
done


###############################################################
# /etc/rc.d/init.d/runfirst
### Fix some things during bootup - run first

cp -a customize/runfirst /etc/init.d/runfirst
chmod +x /etc/rc.d/init.d/runfirst

# runfirst will run at the end of /etc/rc.d/rc.sysinit

# (remove this later)
sed -i "/Executing runfirst/d" /etc/rc.d/rc.sysinit

sysinit_line="/etc/rc.d/init.d/runfirst"
grep -q "$sysinit_line" /etc/rc.d/rc.sysinit
if [ "$?" != "0" ]; then
    echo "$sysinit_line" >> /etc/rc.d/rc.sysinit
    echo >> /etc/rc.d/rc.sysinit
fi


###############################################################
# /etc/rc.d/init.d/runlast
### Fix some things during bootup - run last

cp -a customize/runlast /etc/init.d/runlast
chmod +x /etc/rc.d/init.d/runlast


### Add /etc/rc.d/init.d/runlast to rc.local
LINE=/etc/rc.d/init.d/runlast
grep -q $LINE /etc/rc.d/rc.local
if [ "$?" != "0" ]; then
    # add line
    echo "" >> /etc/rc.d/rc.local
    echo $LINE >> /etc/rc.d/rc.local
    echo "" >> /etc/rc.d/rc.local    
fi


###############################################################
# /etc/sysconfig/networking/devices/ifcfg-eth0
### default eth0 config files
cat > /etc/sysconfig/networking/devices/ifcfg-eth0 <<EOF
DEVICE=eth0
ONBOOT=yes
TYPE=Ethernet
BOOTPROTO=dhcp
USERCTL=no
PEERDNS=yes
IPV6INIT=no
EOF
# make hard links
cp -lf /etc/sysconfig/networking/devices/ifcfg-eth0 /etc/sysconfig/networking/profiles/default/
cp -lf /etc/sysconfig/networking/devices/ifcfg-eth0 /etc/sysconfig/network-scripts/

###############################################################
# /etc/sysconfig/networking/devices/ifcfg-eth1
### default eth1 config files
cat > /etc/sysconfig/networking/devices/ifcfg-eth1 <<EOF
DEVICE=eth1
ONBOOT=yes
TYPE=Ethernet
BOOTPROTO=dhcp
USERCTL=no
PEERDNS=yes
IPV6INIT=no
EOF
# make hard links
cp -lf /etc/sysconfig/networking/devices/ifcfg-eth1 /etc/sysconfig/networking/profiles/default/
cp -lf /etc/sysconfig/networking/devices/ifcfg-eth1 /etc/sysconfig/network-scripts/

echo "done."


###############################################################
# Configure services
###############################################################

echo "Configure services ..."

# off
if [ $PSI ]; then
    chkconfig cfenvd off
    chkconfig cfexecd off
    chkconfig cfservd off
fi
chkconfig microcode_ctl off
chkconfig cups off
chkconfig yum off
chkconfig xinetd off 
chkconfig afs off
chkconfig rhnsd off
chkconfig nfslock off
chkconfig ip6tables off
chkconfig mdmonitor off
chkconfig mdmpd off
chkconfig hpoj off
chkconfig sendmail off
chkconfig sshd off
chkconfig portmap off
chkconfig snmptrapd off
chkconfig snmpd off
chkconfig smartd off
chkconfig cups-config-daemon off

# on
if [ $PSI ]; then
    chkconfig vpnclient_init on
fi
chkconfig apmd on
chkconfig acpid on
chkconfig pcmcia on
chkconfig kudzu on
chkconfig haldaemon on
chkconfig autofs on

echo "done."


###############################################################
# Empty files
###############################################################

echo "Empty files ..."

### /etc/security/users
cat > /etc/security/users <<EOF
EOF

### Files to empty
FILES_TOUCH="/etc/sysconfig/hwconf \
             /etc/resolv.conf \
             /etc/adjtime \
             /etc/modprobe.conf \
             /etc/dhclient-eth0.conf"

for file in $FILES_TOUCH; do
    rm -rf $file
    touch $file
done

echo "done."

###############################################################
